@@ -4,15 +4,15 @@
 Module      :  Network.Browser
 Copyright   :  See LICENSE file
 License     :  BSD
- 
+
 Maintainer  :  Ganesh Sittampalam <ganesh@earth.li>
 Stability   :  experimental
 Portability :  non-portable (not tested)
 
 Session-level interactions over HTTP.
- 
-The "Network.Browser" goes beyond the basic "Network.HTTP" functionality in 
-providing support for more involved, and real, request/response interactions over 
+
+The "Network.Browser" goes beyond the basic "Network.HTTP" functionality in
+providing support for more involved, and real, request/response interactions over
 HTTP. Additional features supported are:
 
 * HTTP Authentication handling
@@ -33,25 +33,25 @@ Example use:
 >               setAllowRedirects True -- handle HTTP redirects
 >               request $ getRequest "http://www.haskell.org/"
 >      return (take 100 (rspBody rsp))
- 
+
 -}
-module Network.Browser 
+module Network.Browser
        ( BrowserState
        , BrowserAction      -- browser monad, effectively a state monad.
        , Proxy(..)
-       
+
        , browse             -- :: BrowserAction a -> IO a
        , request            -- :: Request -> BrowserAction Response
-    
+
        , getBrowserState    -- :: BrowserAction t (BrowserState t)
        , withBrowserState   -- :: BrowserState t -> BrowserAction t a -> BrowserAction t a
-       
+
        , setAllowRedirects  -- :: Bool -> BrowserAction t ()
        , getAllowRedirects  -- :: BrowserAction t Bool
 
        , setMaxRedirects    -- :: Int -> BrowserAction t ()
        , getMaxRedirects    -- :: BrowserAction t (Maybe Int)
-       
+
        , Authority(..)
        , getAuthorities
        , setAuthorities
@@ -59,12 +59,12 @@ module Network.Browser
        , Challenge(..)
        , Qop(..)
        , Algorithm(..)
-       
+
        , getAuthorityGen
        , setAuthorityGen
        , setAllowBasicAuth
        , getAllowBasicAuth
-       
+
        , setMaxErrorRetries  -- :: Maybe Int -> BrowserAction t ()
        , getMaxErrorRetries  -- :: BrowserAction t (Maybe Int)
 
@@ -78,21 +78,21 @@ module Network.Browser
        , getCookieFilter     -- :: BrowserAction t (URI -> Cookie -> IO Bool)
        , defaultCookieFilter -- :: URI -> Cookie -> IO Bool
        , userCookieFilter    -- :: URI -> Cookie -> IO Bool
-       
+
        , Cookie(..)
        , getCookies        -- :: BrowserAction t [Cookie]
        , setCookies        -- :: [Cookie] -> BrowserAction t ()
        , addCookie         -- :: Cookie   -> BrowserAction t ()
-       
+
        , setErrHandler     -- :: (String -> IO ()) -> BrowserAction t ()
        , setOutHandler     -- :: (String -> IO ()) -> BrowserAction t ()
-    
+
        , setEventHandler   -- :: (BrowserEvent -> BrowserAction t ()) -> BrowserAction t ()
-       
+
        , BrowserEvent(..)
        , BrowserEventType(..)
        , RequestID
-       
+
        , setProxy         -- :: Proxy -> BrowserAction t ()
        , getProxy         -- :: BrowserAction t Proxy
 
@@ -100,20 +100,20 @@ module Network.Browser
        , getCheckForProxy -- :: BrowserAction t Bool
 
        , setDebugLog      -- :: Maybe String -> BrowserAction t ()
-       
+
        , getUserAgent     -- :: BrowserAction t String
        , setUserAgent     -- :: String -> BrowserAction t ()
-       
+
        , out              -- :: String -> BrowserAction t ()
        , err              -- :: String -> BrowserAction t ()
        , ioAction         -- :: IO a -> BrowserAction a
 
        , defaultGETRequest
        , defaultGETRequest_
-       
+
        , formToRequest
        , uriDefaultTo
-       
+
          -- old and half-baked; don't use:
        , Form(..)
        , FormVar
@@ -140,13 +140,14 @@ import Control.Monad.Fail
 import Data.Char (toLower)
 import Data.List (isPrefixOf)
 import Data.Maybe (fromMaybe, listToMaybe, catMaybes )
+#if !MIN_VERSION_base(4,8,0)
 import Control.Applicative (Applicative (..), (<$>))
-#ifdef MTL1
-import Control.Monad (filterM, forM_, when, ap)
-#else
-import Control.Monad (filterM, forM_, when)
 #endif
-import Control.Monad.State (StateT (..), MonadIO (..), modify, gets, withStateT, evalStateT, MonadState (..))
+import Control.Monad (filterM, forM_, when)
+import Control.Monad.IO.Class
+   ( MonadIO (..) )
+import Control.Monad.State
+   ( MonadState(..), gets, modify, StateT (..), evalStateT, withStateT )
 
 import qualified System.IO
    ( hSetBuffering, hPutStr, stdout, stdin, hGetChar
@@ -209,7 +210,7 @@ getCookiesFor dom path =
     where
         cookiematch :: Cookie -> Bool
         cookiematch = cookieMatch (dom,path)
-      
+
 
 -- | @setCookieFilter fn@ sets the cookie acceptance filter to @fn@.
 setCookieFilter :: (URI -> Cookie -> IO Bool) -> BrowserAction t ()
@@ -253,7 +254,7 @@ is added to a rejected request this predictive annotation is suppressed.
 
 
 Notes:
- - digest authentication so far ignores qop, so fails to authenticate 
+ - digest authentication so far ignores qop, so fails to authenticate
    properly with qop=auth-int challenges
  - calculates a1 more than necessary
  - doesn't reverse authenticate
@@ -273,7 +274,7 @@ getAuthFor dom pth = getAuthorities >>= return . (filter match)
 
     matchURI :: URI -> Bool
     matchURI s = (uriToAuthorityString s == dom) && (uriPath s `isPrefixOf` pth)
-    
+
 
 -- | @getAuthorities@ return the current set of @Authority@s known
 -- to the browser.
@@ -306,9 +307,9 @@ getAllowBasicAuth :: BrowserAction t Bool
 getAllowBasicAuth = gets bsAllowBasicAuth
 
 -- | @setMaxAuthAttempts mbMax@ sets the maximum number of authentication attempts
--- to do. If @Nothing@, rever to default max.
+-- to do. If @Nothing@, revert to default max.
 setMaxAuthAttempts :: Maybe Int -> BrowserAction t ()
-setMaxAuthAttempts mb 
+setMaxAuthAttempts mb
  | fromMaybe 0 mb < 0 = return ()
  | otherwise          = modify (\ b -> b{bsMaxAuthAttempts=mb})
 
@@ -359,7 +360,7 @@ challengeToAuthority uri ch
   answerable chall       = (chAlgorithm chall) == Just AlgMD5
 
   buildAuth :: Challenge -> String -> String -> Authority
-  buildAuth (ChalBasic r) u p = 
+  buildAuth (ChalBasic r) u p =
        AuthBasic { auSite=uri
                  , auRealm=r
                  , auUsername=u
@@ -410,7 +411,7 @@ data BrowserState connection
       }
 
 instance Show (BrowserState t) where
-    show bs =  "BrowserState { " 
+    show bs =  "BrowserState { "
             ++ shows (bsCookies bs) ("\n"
            {- ++ show (bsAuthorities bs) ++ "\n"-}
             ++ "AllowRedirects: " ++ shows (bsAllowRedirects bs) "} ")
@@ -418,20 +419,12 @@ instance Show (BrowserState t) where
 -- | @BrowserAction@ is the IO monad, but carrying along a 'BrowserState'.
 newtype BrowserAction conn a
  = BA { unBA :: StateT (BrowserState conn) IO a }
-#ifdef MTL1
- deriving (Functor, Monad, MonadIO, MonadState (BrowserState conn))
-
-instance Applicative (BrowserAction conn) where
-  pure  = return
-  (<*>) = ap
-#else
  deriving
  ( Functor, Applicative, Monad, MonadIO, MonadState (BrowserState conn)
 #if MIN_VERSION_base(4,9,0)
  , MonadFail
 #endif
  )
-#endif
 
 runBA :: BrowserState conn -> BrowserAction conn a -> IO a
 runBA bs = flip evalStateT bs . unBA
@@ -441,7 +434,7 @@ runBA bs = flip evalStateT bs . unBA
 browse :: BrowserAction conn a -> IO a
 browse = runBA defaultBrowserState
 
--- | The default browser state has the settings 
+-- | The default browser state has the settings
 defaultBrowserState :: BrowserState t
 defaultBrowserState = res
  where
@@ -463,7 +456,7 @@ defaultBrowserState = res
      , bsConnectionPool   = []
      , bsCheckProxy       = defaultAutoProxyDetect
      , bsProxy            = noProxy
-     , bsDebug            = Nothing 
+     , bsDebug            = Nothing
      , bsEvent            = Nothing
      , bsRequestID        = 0
      , bsUserAgent        = Nothing
@@ -484,8 +477,8 @@ withBrowserState bs = BA . withStateT (const bs) . unBA
 -- before doing so.
 nextRequest :: BrowserAction t a -> BrowserAction t a
 nextRequest act = do
-  let updReqID st = 
-       let 
+  let updReqID st =
+       let
         rid = succ (bsRequestID st)
        in
        rid `seq` st{bsRequestID=rid}
@@ -522,13 +515,13 @@ setAllowRedirects bl = modify (\b -> b {bsAllowRedirects=bl})
 getAllowRedirects :: BrowserAction t Bool
 getAllowRedirects = gets bsAllowRedirects
 
--- | @setMaxRedirects maxCount@ sets the maxiumum number of forwarding hops
+-- | @setMaxRedirects maxCount@ sets the maximum number of forwarding hops
 -- we are willing to jump through. A no-op if the count is negative; if zero,
 -- the max is set to whatever default applies. Notice that setting the max
 -- redirects count does /not/ enable following of redirects itself; use
 -- 'setAllowRedirects' to do so.
 setMaxRedirects :: Maybe Int -> BrowserAction t ()
-setMaxRedirects c 
+setMaxRedirects c
  | fromMaybe 0 c < 0  = return ()
  | otherwise          = modify (\b -> b{bsMaxRedirects=c})
 
@@ -550,7 +543,7 @@ getMaxPoolSize = gets bsMaxPoolSize
 
 -- | @setProxy p@ will disable proxy usage if @p@ is @NoProxy@.
 -- If @p@ is @Proxy proxyURL mbAuth@, then @proxyURL@ is interpreted
--- as the URL of the proxy to use, possibly authenticating via 
+-- as the URL of the proxy to use, possibly authenticating via
 -- 'Authority' information in @mbAuth@.
 setProxy :: Proxy -> BrowserAction t ()
 setProxy p =
@@ -560,7 +553,7 @@ setProxy p =
 
 -- | @getProxy@ returns the current proxy settings. If
 -- the auto-proxy flag is set to @True@, @getProxy@ will
--- perform the necessary 
+-- perform the necessary
 getProxy :: BrowserAction t Proxy
 getProxy = do
   p <- gets bsProxy
@@ -571,7 +564,7 @@ getProxy = do
     NoProxy{} -> do
      flg <- gets bsCheckProxy
      if not flg
-      then return p 
+      then return p
       else do
        np <- liftIO $ fetchProxy True{-issue warning on stderr if ill-formed...-}
         -- note: this resets the check-proxy flag; a one-off affair.
@@ -607,7 +600,7 @@ setDebugLog v = modify (\b -> b {bsDebug=v})
 -- A common form of user agent string is @\"name\/version (details)\"@. For
 -- example @\"cabal-install/0.10.2 (HTTP 4000.1.2)\"@. Including the version
 -- of this HTTP package can be helpful if you ever need to track down HTTP
--- compatability quirks. This version is available via 'httpPackageVersion'.
+-- compatibility quirks. This version is available via 'httpPackageVersion'.
 -- For more info see <http://en.wikipedia.org/wiki/User_agent>.
 --
 setUserAgent :: String -> BrowserAction t ()
@@ -619,10 +612,10 @@ getUserAgent  = do
   n <- gets bsUserAgent
   return (maybe defaultUserAgent id n)
 
--- | @RequestState@ is an internal tallying type keeping track of various 
--- per-connection counters, like the number of authorization attempts and 
+-- | @RequestState@ is an internal tallying type keeping track of various
+-- per-connection counters, like the number of authorization attempts and
 -- forwards we've gone through.
-data RequestState 
+data RequestState
   = RequestState
       { reqDenies     :: Int   -- ^ number of 401 responses so far
       , reqRedirects  :: Int   -- ^ number of redirects so far
@@ -665,7 +658,7 @@ data BrowserEventType
  | AuthChallenge
  | AuthResponse
 -}
- 
+
 -- | @setEventHandler onBrowserEvent@ configures event handling.
 -- If @onBrowserEvent@ is @Nothing@, event handling is turned off;
 -- setting it to @Just onEv@ causes the @onEv@ IO action to be
@@ -677,7 +670,7 @@ setEventHandler mbH = modify (\b -> b { bsEvent=mbH})
 buildBrowserEvent :: BrowserEventType -> {-URI-}String -> RequestID -> IO BrowserEvent
 buildBrowserEvent bt uri reqID = do
   ct <- getCurrentTime
-  return BrowserEvent 
+  return BrowserEvent
          { browserTimestamp  = ct
          , browserRequestID  = reqID
          , browserRequestURI = uri
@@ -693,7 +686,7 @@ reportEvent bt uri = do
        evt <- liftIO $ buildBrowserEvent bt uri (bsRequestID st)
        evH evt -- if it fails, we fail.
 
--- | The default number of hops we are willing not to go beyond for 
+-- | The default number of hops we are willing not to go beyond for
 -- request forwardings.
 defaultMaxRetries :: Int
 defaultMaxRetries = 4
@@ -733,7 +726,7 @@ request req = nextRequest $ do
   initialState = nullRequestState
   nullVal      = buf_empty bufferOps
 
--- | Internal helper function, explicitly carrying along per-request 
+-- | Internal helper function, explicitly carrying along per-request
 -- counts.
 request' :: HStream ty
          => ty
@@ -743,7 +736,7 @@ request' :: HStream ty
 request' nullVal rqState rq = do
    let uri = rqURI rq
    failHTTPS uri
-   let uria = reqURIAuth rq 
+   let uria = reqURIAuth rq
      -- add cookies to request
    cookies <- getCookiesFor (uriAuthToString uria) (uriPath uri)
 {- Not for now:
@@ -752,20 +745,20 @@ request' nullVal rqState rq = do
      xs ->
        case chopAtDelim ':' xs of
          (_,[])    -> id
-	 (usr,pwd) -> withAuth
-	                  AuthBasic{ auUserName = usr
+         (usr,pwd) -> withAuth
+                          AuthBasic{ auUserName = usr
                                    , auPassword = pwd
-			           , auRealm    = "/"
-			           , auSite     = uri
-			           }) $ do
+                                   , auRealm    = "/"
+                                   , auSite     = uri
+                                   }) $ do
 -}
-   when (not $ null cookies) 
+   when (not $ null cookies)
         (out $ "Adding cookies to request.  Cookie names: "  ++ unwords (map ckName cookies))
     -- add credentials to request
-   rq' <- 
-    if not (reqStopOnDeny rqState) 
-     then return rq 
-     else do 
+   rq' <-
+    if not (reqStopOnDeny rqState)
+     then return rq
+     else do
        auth <- anticipateChallenge rq
        case auth of
          Nothing -> return rq
@@ -774,7 +767,7 @@ request' nullVal rqState rq = do
    p <- getProxy
    def_ua <- gets bsUserAgent
    let defaultOpts =
-         case p of 
+         case p of
            NoProxy     -> defaultNormalizeRequestOptions{normUserAgent=def_ua}
            Proxy _ ath ->
               defaultNormalizeRequestOptions
@@ -787,11 +780,11 @@ request' nullVal rqState rq = do
                 }
    let final_req = normalizeRequest defaultOpts rq''
    out ("Sending:\n" ++ show final_req)
-   e_rsp <- 
+   e_rsp <-
      case p of
        NoProxy        -> dorequest (reqURIAuth rq'') final_req
        Proxy str _ath -> do
-          let notURI 
+          let notURI
                | null pt || null hst =
                  URIAuth{ uriUserInfo = ""
                         , uriRegName  = str
@@ -815,18 +808,18 @@ request' nullVal rqState rq = do
           dorequest proxyURIAuth final_req
    mbMx <- getMaxErrorRetries
    case e_rsp of
-    Left v 
-     | (reqRetries rqState < fromMaybe defaultMaxErrorRetries mbMx) && 
+    Left v
+     | (reqRetries rqState < fromMaybe defaultMaxErrorRetries mbMx) &&
        (v == ErrorReset || v == ErrorClosed) -> do
        --empty connnection pool in case connection has become invalid
-       modify (\b -> b { bsConnectionPool=[] })       
+       modify (\b -> b { bsConnectionPool=[] })
        request' nullVal rqState{reqRetries=succ (reqRetries rqState)} rq
-     | otherwise -> 
+     | otherwise ->
        return (Left v)
-    Right rsp -> do 
+    Right rsp -> do
      out ("Received:\n" ++ show rsp)
       -- add new cookies to browser state
-     handleCookies uri (uriAuthToString $ reqURIAuth rq) 
+     handleCookies uri (uriAuthToString $ reqURIAuth rq)
                        (retrieveHeaders HdrSetCookie rsp)
      -- Deal with "Connection: close" in response.
      handleConnectionClose (reqURIAuth rq) (retrieveHeaders HdrConnection rsp)
@@ -894,10 +887,10 @@ request' nullVal rqState rq = do
           False -> return (Right (uri,rsp))
           _ -> do
            case retrieveHeaders HdrLocation rsp of
-            [] -> do 
+            [] -> do
               err "No Location: header in redirect response"
               return (Right (uri,rsp))
-            (Header _ u:_) -> 
+            (Header _ u:_) ->
               case parseURIReference u of
                 Nothing -> do
                   err ("Parse of Location: header in a redirect response failed: " ++ u)
@@ -908,14 +901,14 @@ request' nullVal rqState rq = do
                     return (Right (uri, rsp))
                  | otherwise -> do
                     out ("Redirecting to " ++ show newURI_abs ++ " ...")
-                    
+
                     -- Redirect using GET request method, depending on
                     -- response code.
                     let toGet = x `elem` [2,3]
                         method = if toGet then GET else rqMethod rq
                         rq1 = rq { rqMethod=method, rqURI=newURI_abs }
                         rq2 = if toGet then (replaceHeader HdrContentLength "0") (rq1 {rqBody = nullVal}) else rq1
-                    
+
                     request' nullVal
                             rqState{ reqDenies     = 0
                                    , reqRedirects  = succ(reqRedirects rqState)
@@ -927,10 +920,10 @@ request' nullVal rqState rq = do
 
       (3,0,5) ->
         case retrieveHeaders HdrLocation rsp of
-         [] -> do 
+         [] -> do
            err "No Location header in proxy redirect response."
            return (Right (uri,rsp))
-         (Header _ u:_) -> 
+         (Header _ u:_) ->
            case parseURIReference u of
             Nothing -> do
              err ("Parse of Location header in a proxy redirect response failed: " ++ u)
@@ -956,9 +949,9 @@ dorequest hst rqst = do
   pool <- gets bsConnectionPool
   let uPort = uriAuthPort Nothing{-ToDo: feed in complete URL-} hst
   conn <- liftIO $ filterM (\c -> c `isTCPConnectedTo` EndPoint (uriRegName hst) uPort) pool
-  rsp <- 
+  rsp <-
     case conn of
-      [] -> do 
+      [] -> do
         out ("Creating new connection to " ++ uriAuthToString hst)
         reportEvent OpenConnection (show (rqURI rqst))
         c <- liftIO $ openStream (uriRegName hst) uPort
@@ -968,15 +961,15 @@ dorequest hst rqst = do
         out ("Recovering connection to " ++ uriAuthToString hst)
         reportEvent ReuseConnection (show (rqURI rqst))
         dorequest2 c rqst
-  case rsp of 
-     Right (Response a b c _) -> 
+  case rsp of
+     Right (Response a b c _) ->
          reportEvent (ResponseEnd (a,b,c)) (show (rqURI rqst)) ; _ -> return ()
   return rsp
  where
   dorequest2 c r = do
     dbg <- gets bsDebug
     st  <- get
-    let 
+    let
      onSendComplete =
        maybe (return ())
              (\evh -> do
@@ -984,7 +977,7 @@ dorequest hst rqst = do
                 runBA st (evh x)
                 return ())
              (bsEvent st)
-    liftIO $ 
+    liftIO $
       maybe (sendHTTP_notify c r onSendComplete)
             (\ f -> do
                c' <- debugByteStream (f++'-': uriAuthToString hst) c
@@ -1000,12 +993,12 @@ updateConnectionPool c = do
    maxPoolSize <- fromMaybe defaultMaxPoolSize <$> gets bsMaxPoolSize
    when (len_pool > maxPoolSize)
         (liftIO $ close (last pool))
-   let pool' 
+   let pool'
         | len_pool > maxPoolSize = init pool
         | otherwise              = pool
    when (maxPoolSize > 0) $ modify (\b -> b { bsConnectionPool=c:pool' })
    return ()
-                             
+
 -- | Default maximum number of open connections we are willing to have active.
 defaultMaxPoolSize :: Int
 defaultMaxPoolSize = 5
@@ -1096,4 +1089,3 @@ formToRequest (Form m u vs) =
                         }
         _ -> error ("unexpected request: " ++ show m)
 
-
